<?php

namespace Drupal\block_usage_report\Controller;

use Drupal\block\Entity\Block;
use Drupal\block_content\Entity\BlockContent;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\ContentEntityInterface;

/**
 * Returns output for the block usage report.
 */
class BlockUsageReportController extends ControllerBase {

  /**
   * Finds a content entity referenced by a UUID, given a collection.
   *
   * @param string $uuid
   *   The UUID to search for.
   * @param \Drupal\Core\Entity\ContentEntityInterface[] $entities
   *   The collection of entities to search for.
   *
   * @return \Drupal\Core\Entity\ContentEntityInterface|null
   *   THe matching entity, or NULL if not found.
   */
  protected function getEntityByUuid(string $uuid, array $entities): ?ContentEntityInterface {
    foreach ($entities as $entity) {
      if ($entity->uuid() == $uuid) {
        return $entity;
      }
    }
    return NULL;
  }

  /**
   * Builds the output of the report.
   *
   * @return array
   *   The render-array for the report.
   */
  public function build(): array {
    $bc_blocks = BlockContent::loadMultiple();
    $bc_uuids_found = [];

    /** @var \Drupal\block\Entity\Block[] $blocks */
    $blocks = Block::loadMultiple();
    $public_theme_name = $this->config('system.theme')->get('default');

    $header = [
      $this->t('Name'),
      $this->t('Provider'),
      $this->t('Type'),
      $this->t('Region'),
      $this->t('Operations'),
    ];

    $enabled_rows = $disabled_rows = [];

    $block_list_builder = $this->entityTypeManager()->getListBuilder('block');
    $bc_list_builder = $this->entityTypeManager()->getListBuilder('block_content');

    foreach ($blocks as $block) {
      $block_theme = $block->getTheme();
      if ($block_theme != $public_theme_name) {
        continue;
      }
      $settings = $block->get('settings');
      $name = $block->label() . ' (' . $block->id() . ')';
      $provider = $settings['provider'];
      $region = $block->getRegion();
      $status = $block->status();
      $plugin_id = $block->getPluginId();
      if ($provider == 'block_content') {
        $uuid = explode(':', $plugin_id)[1];
        $block_content = $this->getEntityByUuid($uuid, $bc_blocks);
        $type = $block_content ? $block_content->bundle() : $this->t('[unknown]');
        $bc_uuids_found[] = $uuid;
      }
      elseif (str_contains($plugin_id, ':')) {
        $type = explode(':', $plugin_id)[1];
      }
      else {
        $type = '';
      }

      $row = [
        $name,
        $provider,
        $type,
        $region,
        [
          'data' => [
            '#type' => 'operations',
            '#links' => $block_list_builder->getOperations($block),
          ],
        ],
      ];

      if ($status) {
        $enabled_rows[] = $row;
      }
      else {
        $disabled_rows[] = $row;
      }

    }

    $sorter = function ($a, $b) {
      if ($a[1] != $b[1]) {
        return $a[1] <=> $b[1];
      }
      if ($a[2] != $b[2]) {
        return $a[2] <=> $b[2];
      }
      return $a[0] <=> $b[0];
    };

    usort($enabled_rows, $sorter);
    usort($disabled_rows, $sorter);

    $unplaced_blocks = [];
    foreach ($bc_blocks as $bc_block) {
      if (!in_array($bc_block->uuid(), $bc_uuids_found)) {
        $unplaced_blocks[] = [
          $bc_block->label() . ' (' . $bc_block->id() . ')',
          $bc_block->bundle(),
          [
            'data' => [
              '#type' => 'operations',
              '#links' => $bc_list_builder->getOperations($bc_block),
            ],
          ],
        ];
      }
    }
    if (!empty($unplaced_blocks)) {
      usort($unplaced_blocks, function ($a, $b) {
        if ($a[1] != $b[1]) {
          return $a[1] <=> $b[1];
        }
        return $a[0] <=> $b[0];
      });
    }

    $out = [];
    if (!empty($enabled_rows)) {
      $out['enabled_wrapper'] = [
        '#type' => 'details',
        '#title' => $this->t('Enabled blocks'),
        '#open' => FALSE,
      ];
      $out['enabled_wrapper']['enabled'] = [
        '#theme' => 'table',
        '#header' => $header,
        '#rows' => $enabled_rows,
      ];
    }
    if (!empty($disabled_rows)) {
      $out['disbled_wrapper'] = [
        '#type' => 'details',
        '#title' => $this->t('Disabled blocks'),
        '#open' => FALSE,
      ];
      $out['disbled_wrapper']['disabled'] = [
        '#theme' => 'table',
        '#header' => $header,
        '#rows' => $disabled_rows,
      ];
    }
    if (!empty($unplaced_blocks)) {
      $out['unplaced_wrapper'] = [
        '#type' => 'details',
        '#title' => $this->t('Unplaced custom blocks'),
        '#open' => FALSE,
      ];
      $out['unplaced_wrapper']['unplaced'] = [
        '#theme' => 'table',
        '#header' => [
          $this->t('Name'),
          $this->t('Type'),
          $this->t('Operations'),
        ],
        '#rows' => $unplaced_blocks,
      ];
    }
    return $out;

  }

}
